@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// issues on https://gitlab.onelab.info/gmsh/gmsh/issues
+// issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 
 #include <sstream>
 #include "GModel.h"
@@ -32,7 +32,7 @@ bool gmshEdge::degenerate(int dim) const
 {
   if(c->beg == c->end && c->Typ == MSH_SEGM_LINE &&
      List_Nbr(c->Control_Points) == 0) {
-    Msg::Info("Model Edge %d is degenerate", tag());
+    Msg::Info("Curve %d is degenerate", tag());
     return true;
   }
   return false;
